<?php
/**
 * -----------------------------------------------------------------------
 * vBSSO is a plugin that helps you connect to different software platforms
 * via secure Single Sign-On.
 *
 * Copyright (c) 2011-2012 vBSSO. All Rights Reserved.
 * This software is the proprietary information of vBSSO.
 *
 * Author URI: http://www.vbsso.com
 * License: GPL version 2 or later - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * -----------------------------------------------------------------------
 * $Revision: 714 $:
 * $Date: 2012-09-13 13:19:14 +0300 (Чт, 13 сен 2012) $:
 */

define ('VBSSO_WORDPRESS_PLUGIN_VERSION', '1.2.4');
if ($wp_version <= '3.0.6')
    require_once(ABSPATH . 'wp-includes/registration.php');

function vbsso_listener_report_error($error) {
    if (vbsso_get_platform_config_property(SHAREDAPI_PLATFORM_WORDPRESS, 'log', false)) {
        //        TODO: WordPress has no it's own logging. Available plugins are http://wordpress.org/extend/plugins/tags/logging. Connect to one of that plugins later.
    }

    $code = !is_string($error) ? $error->get_error_code() : '';
    $message = !is_string($error) ? $error->get_error_message($code) : $error;
    $data = !is_string($error) ? $error->get_error_data($code) : '';

    return array(SHAREDAPI_EVENT_FIELD_ERROR_CODE => $code, SHAREDAPI_EVENT_FIELD_ERROR_MESSAGE => $message, SHAREDAPI_EVENT_FIELD_ERROR_DATA => $data);
}

//function vbsso_listener_user_generate_username($username, $email) {
//    $new_username = $username;
//
//    $user = get_userdatabylogin($new_username);
//    if ($user) {
//        $new_username = $email;
//
//        $user = get_userdatabylogin($new_username);
//        if ($user) {
//            $new_username = $username . '_' . $email;
//
//            $user = get_userdatabylogin($new_username);
//            if ($user) {
//                $index = 2;
//
//                $new_username = $username . $index;
//                while (get_userdatabylogin($new_username)) {
//                    $index++;
//                    $new_username = $username . $index;
//                }
//            }
//        }
//    }
//
//    return $new_username;
//}

function vbsso_listener_user_load($json, $create_user = false) {
    $user = get_user_by('email', $json[SHAREDAPI_EVENT_FIELD_EMAIL]);
    if (!$user && $create_user) {
//        $username = vbsso_listener_user_generate_username($json[SHAREDAPI_EVENT_FIELD_USERNAME], $json[SHAREDAPI_EVENT_FIELD_EMAIL]);
        $username = html_entity_decode($json[SHAREDAPI_EVENT_FIELD_USERNAME], null, get_option('blog_charset'));
        $user_id = wp_create_user($username, '', $json[SHAREDAPI_EVENT_FIELD_EMAIL]);

        if (!is_wp_error($user_id)) {
            $user = get_userdata($user_id);

            $data = array('user_login' => $username, 'user_nicename' => $username);
            $data = stripslashes_deep($data);

            global $wpdb;
            $res = $wpdb->update($wpdb->users, $data, array('ID' => $user->ID));
            if ($res === false) {
                $user = vbsso_wp_create_error($json, 'unable_to_update_user_login', 'Unable to update user login');
            }
        } else {
            return vbsso_listener_report_error($user_id);
        }
    }

    return $user ? $user : vbsso_listener_report_error('Unable to load user: ' . $json[SHAREDAPI_EVENT_FIELD_EMAIL]);
}

sharedapi_data_handler(SHAREDAPI_PLATFORM_WORDPRESS, $wp_version,
    VBSSO_WORDPRESS_PLUGIN_VERSION,
    get_site_option(VBSSO_NAMED_EVENT_FIELD_API_KEY, SHAREDAPI_DEFAULT_API_KEY),
    array(
        SHAREDAPI_EVENT_VERIFY => 'vbsso_listener_verify',
        SHAREDAPI_EVENT_LOGIN => 'vbsso_listener_register',
        SHAREDAPI_EVENT_AUTHENTICATION => 'vbsso_listener_authentication',
        SHAREDAPI_EVENT_LOGOUT => 'vbsso_listener_logout',
        SHAREDAPI_EVENT_REGISTER => 'vbsso_listener_register',
        SHAREDAPI_EVENT_CREDENTIALS => 'vbsso_listener_credentials',
));

function vbsso_listener_verify($json) {
    $supported = vbsso_get_supported_api_properties();
    foreach ($supported as $key => $item) {
        if (get_site_option($key) != $json[$item['field']]) {
            update_site_option($key, $json[$item['field']]);
        }
    }

    return array('data' => array(SHAREDAPI_EVENT_FIELD_VERIFY => true));
}

function vbsso_listener_authentication($json) {
    $user = wp_get_current_user(); // object exists for both guest and authenticated user always.

    $u = vbsso_listener_user_load($json, true);
    if (!sharedapi_is_error_data_item($u)) {
        if ($user->ID != $u->ID) {
            vbsso_listener_logout($json);

            setcookie(VBSSO_NAMED_EVENT_FIELD_TIMEOUT, $json[VBSSO_NAMED_EVENT_FIELD_TIMEOUT], 0, '/', '/');

            wp_set_current_user($u->ID);
            wp_set_auth_cookie($u->ID, isset($json[SHAREDAPI_EVENT_FIELD_REMEMBERME]) && $json[SHAREDAPI_EVENT_FIELD_REMEMBERME]);
            do_action('wp_login', $u->user_login);
        }
    } else {
        return array('error' => $u);
    }
}

function vbsso_listener_logout($json) {
    if (is_user_logged_in()) {
        wp_logout();
    }
}

function vbsso_listener_register($json) {
    $u = vbsso_listener_user_load($json, true);

    if (sharedapi_is_error_data_item($u)) {
        return array('error' => $u);
    }
}

function vbsso_listener_credentials($json) {
    $u = vbsso_listener_user_load($json, false);
    if (!sharedapi_is_error_data_item($u)) {
        $update = false;

        if (isset($json[SHAREDAPI_EVENT_FIELD_EMAIL2])) {
            $u->user_email = $json[SHAREDAPI_EVENT_FIELD_EMAIL2];
            $update = true;
        }

        if (isset($json[SHAREDAPI_EVENT_FIELD_USERNAME2])) {
//            $u->user_nicename = $u->user_login = vbsso_listener_user_generate_username($json[SHAREDAPI_EVENT_FIELD_USERNAME2], $u->user_email);
            $u->user_nicename = $u->user_login = html_entity_decode($json[SHAREDAPI_EVENT_FIELD_USERNAME2], null, get_option('blog_charset'));
            $update = true;
        }

        if ($update) {
            if (get_class($u) == 'WP_User') {
                $user_id = wp_insert_user(get_object_vars($u->data));
            } else {
                $user_id = wp_insert_user(get_object_vars($u));
            }

            if (!is_wp_error($user_id)) {
                $u = get_userdata($user_id);

                if ($u->user_login != $u->user_nicename) {
                    $data = array('user_login' => $u->user_nicename);
                    $data = stripslashes_deep($data);

                    global $wpdb;
                    $res = $wpdb->update($wpdb->users, $data, array('ID' => $u->ID));
                    if ($res === false) {
                        return array('error' => vbsso_listener_report_error('Unable to update user login: ' . $u->user_login));
                    }
                }
            } else {
                return array('error' => vbsso_listener_report_error($user_id));
            }
        }
    } else {
        return array('error' => $u);
    }
}